###################################################################################################
# _copysupport.py
#
# Implementation of class CopySupport (see below).
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
###################################################################################################

# Imports.
import string 
import urllib
from OFS import Moniker
from OFS.CopySupport import _cb_decode, _cb_encode, absattr, CopyError, eNoData, eInvalid
# Product Imports.
import _globals


###################################################################################################
###################################################################################################
###
###   C l a s s   C o p y S u p p o r t 
###
###################################################################################################
###################################################################################################
class CopySupport:

    # ------------------------------------------------------------------------------------------
    # CopySupport._get_cb_copy_data:
    # ------------------------------------------------------------------------------------------
    def _get_cb_copy_data(self, cb_copy_data=None, REQUEST=None):
      cp=None
      if cb_copy_data is not None:
        cp=cb_copy_data
      else:
        if REQUEST and REQUEST.has_key('__cp'):
          cp=REQUEST['__cp']
      if cp is None:
        raise CopyError, eNoData
      
      try: cp=_cb_decode(cp)
      except: raise CopyError, eInvalid
      
      return cp

    # ------------------------------------------------------------------------------------------
    # CopySupport._get_obs:
    # ------------------------------------------------------------------------------------------
    def _get_obs(self, cp):
        
        try: cp=_cb_decode(cp)
        except: raise CopyError, eInvalid
        
        oblist=[]
        op=cp[0]
        app = self.getPhysicalRoot()
        
        for mdata in cp[1]:
          m = Moniker.loadMoniker(mdata)
          try: ob = m.bind(app)
          except: raise CopyError, eNotFound
          self._verifyObjectPaste(ob)
          oblist.append(ob)
        
        return oblist

    # ------------------------------------------------------------------------------------------
    # CopySupport._get_ids:
    # ------------------------------------------------------------------------------------------
    def _get_ids(self, cp):
      oblist=self._get_obs(cp)
      rtn = []
      for ob in oblist:
        rtn.append(self._get_id(absattr(ob.id)))
      return rtn

    # ------------------------------------------------------------------------------------------
    # CopySupport._get_id:
    #
    # Allow containers to override the generation of
    # object copy id by attempting to call its _get_id
    # method, if it exists.
    # ------------------------------------------------------------------------------------------
    def _get_id(self, id):
      newId = 'copy_of_%s'%id 
      return newId

    # ------------------------------------------------------------------------------------------
    # CopySupport._set_sort_ids:
    # ------------------------------------------------------------------------------------------
    def _set_sort_ids(self, ids, REQUEST):
      sort_id = REQUEST['_sort_id']
      for ob in self.getChildNodes():
        id = absattr(ob.id)
        if id in ids:
          sort_id = sort_id + 1
          ob.setObjProperty('sort_id',_globals.format_sort_id(sort_id))
          ob.onChangeObj(REQUEST,forced=1)

    # ------------------------------------------------------------------------------------------
    # CopySupport._normalize_ids_after_copy:
    # ------------------------------------------------------------------------------------------
    def _normalize_ids_after_copy(self, ids=[], forced=0, REQUEST={}):
      for ob in self.getChildNodes():
        id = absattr(ob.id)
        if id in ids or forced:
          if _globals.debug: print '[%s._normalize_ids_after_copy]: %s.%s(%s)'%(self.meta_type,self.id,id,ob.meta_type)
          
          # Assign new id.
          copy_of_prefix = 'copy_of_'
          id_prefix = _globals.id_prefix(id)
          if string.find(id_prefix,copy_of_prefix)==0:
            id_prefix = id_prefix[len(copy_of_prefix):]
          if id_prefix != id:
            new_id = self.getNewId(id_prefix)
            self.manage_renameObject(id=id,new_id=new_id)

          for s_lang in self.getLanguages():
            for s_preview in ['preview']:
              req = {'lang':s_lang,'preview':s_preview,'AUTHENTICATED_USER':REQUEST.get('AUTHENTICATED_USER','Anonymous')}
              if string.find(id,'copy_of_') == 0:
                # Object-State and Version-Manager.
                ob.setObjStateNew(req,reset=0)
                ob.onChangeObj(req)
                # Unlock.
                ob.checkIn(req)
                # Process referential integrity.
                ob.onCopyRefObj(req)
          
          # Process tree.
          ob._normalize_ids_after_copy(ids=ids,forced=1,REQUEST=REQUEST)


    # ------------------------------------------------------------------------------------------
    # CopySupport._normalize_ids_after_move:
    # ------------------------------------------------------------------------------------------
    def _normalize_ids_after_move(self, ids=[], forced=0, REQUEST={}):
      for ob in self.getChildNodes():
        id = absattr(ob.id)
        if id in ids or forced:
          if _globals.debug: print '[%s._normalize_ids_after_move]: %s.%s(%s)'%(self.meta_type,self.id,id,ob.meta_type)
          
          # Re-Assign old id.
          copy_of_prefix = 'copy_of_'
          if string.find(id,copy_of_prefix) == 0:
            new_id = id[len(copy_of_prefix):]
            self.manage_renameObject(id=id,new_id=new_id)
            
          for s_lang in self.getLanguages():
            for s_preview in ['preview']:
              req = {'lang':s_lang,'preview':s_preview,'AUTHENTICATED_USER':REQUEST.get('AUTHENTICATED_USER','Anonymous')}
              if string.find(id,'copy_of_') == 0:
                # Object-State and Version-Manager.
                ob.setObjStateModified(req)
                ob.onChangeObj(req)
                # Process referential integrity.
                ob.onMoveRefObj(req)
          
          # Process tree.
          ob._normalize_ids_after_move(ids=ids,forced=1,REQUEST=REQUEST)


    ###############################################################################################
    # CopySupport.manage_pasteObjs: 
    #
    # Paste previously copied objects into the current object.
    # If calling manage_pasteObjects from python code, pass
    # the result of a previous call to manage_cutObjects or
    # manage_copyObjects as the first argument.
    ###############################################################################################
    def manage_pasteObjs(self, REQUEST, RESPONSE=None):
        """ CopySupport.manage_pasteObjs """
        cp=self._get_cb_copy_data(cb_copy_data=None,REQUEST=REQUEST)
        op=cp[0]
        
        # Copy objects.
        cp = (0,cp[1])
        cp = _cb_encode(cp)
        ids = self._get_ids(cp)
        oblist = self._get_obs(cp)
        
        # Change sort order.
        resort = op == 1 and len(oblist) > 0 and oblist[0].id in self.objectIds()
        
        if not resort:
          self.manage_pasteObjects(cp)
        else:
          ids = [] 
          for ob in oblist:
            ids.append(ob.id)
        
        self._set_sort_ids(ids=ids,REQUEST=REQUEST)
        self.normalizeSortIds()
        
        # Move: delete objects.
        if op==1:
          if not resort:
            self._normalize_ids_after_move(ids=ids,forced=0,REQUEST=REQUEST)
            for ob in oblist:
              ob.aq_parent.manage_delObjects(ids=[absattr(ob.id)])
        # Copy.
        else:
          self._normalize_ids_after_copy(ids=ids,forced=0,REQUEST=REQUEST)
        
        # Return with message.
        if RESPONSE is not None:
          message = urllib.quote(self.getLangStr('MSG_CLIPBOARD_PASTE',REQUEST['manage_lang']))
          RESPONSE.redirect('manage_main?lang=%s&manage_lang=%s&manage_tabs_message=%s'%(REQUEST['lang'],REQUEST['manage_lang'],message))

###################################################################################################